"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore
const fetch_undici_1 = require("fetch-undici");
const debug_1 = __importDefault(require("debug"));
const is_buffer_1 = __importDefault(require("is-buffer"));
const debug = (0, debug_1.default)('sonar:fetch');
async function makeFetch(url, opts) {
    if (!url.match(/^https?:\/\//)) {
        if (url.includes('://')) {
            throw new Error('Only http: and https: protocols are supported.');
        }
        if (!url.startsWith('/')) {
            url = '/' + url;
        }
        if (opts.endpoint) {
            url = opts.endpoint + url;
        }
    }
    if (!opts.headers)
        opts.headers = {};
    if (!opts.requestType) {
        if ((0, is_buffer_1.default)(opts.body)) {
            opts.requestType = 'buffer';
        }
        else {
            opts.requestType = 'json';
        }
    }
    if (opts.params) {
        const searchParams = new URLSearchParams();
        for (const [key, value] of Object.entries(opts.params)) {
            searchParams.append(key, value);
        }
        url += '?' + searchParams.toString();
    }
    if (opts.requestType === 'json') {
        opts.body = JSON.stringify(opts.body);
        opts.headers['content-type'] = 'application/json';
    }
    if (opts.requestType === 'buffer') {
        opts.headers['content-type'] = 'application/octet-stream';
    }
    const log = opts.log || debug;
    try {
        debug('fetch', url, opts);
        const res = await (0, fetch_undici_1.fetch)(url, opts);
        if (!res.ok) {
            let message;
            if (isJsonResponse(res)) {
                message = (await res.json()).error;
            }
            else {
                message = await res.text();
            }
            log(`error ${res.status} ${url}`);
            throw new Error('Remote error (code ' + res.status + '): ' + message);
        }
        log(`ok ${res.status} fetch ${url}`);
        if (opts.responseType === 'raw') {
            return res;
        }
        if (opts.responseType === 'stream') {
            return res.body;
        }
        if (opts.responseType === 'buffer') {
            const buf = await res.arrayBuffer();
            return new Uint8Array(buf);
        }
        if (isJsonResponse(res)) {
            return await res.json();
        }
        return await res.text();
    }
    catch (err) {
        // TODO: If error fails for insufficient authorization, try creating
        // a new token if accessCode is set
        debug('fetch error', err);
        log(`error: fetch ${url} ${err.message}`);
        throw err;
    }
}
exports.default = makeFetch;
function isJsonResponse(res) {
    const header = res.headers.get('content-type');
    if (!header) {
        return false;
    }
    return header.includes('application/json');
}
//# sourceMappingURL=fetch.js.map