"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tape_1 = __importDefault(require("tape"));
const create_js_1 = require("./lib/create.js");
(0, tape_1.default)('sync', async (t) => {
    const { client, cleanup } = await (0, create_js_1.createOne)();
    const collection = await client.createCollection('sync');
    try {
        await collection.putType({
            name: 'foo',
            fields: {
                title: { type: 'string' }
            }
        });
        const { id } = await collection.put({
            type: 'foo',
            id: 'bar',
            value: { title: 'bar' }
        });
        await collection.sync();
        const records = await collection.get({ id });
        t.equal(records.length, 1);
        t.equal(records[0].id, 'bar');
    }
    catch (err) {
        t.fail(String(err));
    }
    await cleanup();
});
(0, tape_1.default)('events', async (t) => {
    const { client, cleanup } = await (0, create_js_1.createOne)();
    try {
        const collection = await client.createCollection('test');
        const eventStream = collection.createEventStream();
        const events = [];
        eventStream.on('data', event => {
            events.push(event);
        });
        await collection.putType({
            name: 'foo',
            fields: { title: { type: 'string' } }
        });
        // console.log('type created')
        await collection.put({ type: 'foo', value: { title: 'hello world' } });
        // console.log('record created')
        await collection.sync();
        // Check that at least one schema-update and update event was received.
        const eventTypes = events.map(e => e.event);
        t.ok(eventTypes.includes('update'), 'update event received');
        t.ok(eventTypes.includes('schema-update'), 'schema update update event received');
    }
    catch (e) {
        console.log(e);
        throw e;
    }
    await cleanup();
    t.end();
});
//# sourceMappingURL=sync.js.map